<?php

namespace Ecommerce\Services;

use Doctrine\DBAL\Connection;

class WebServiceAFService
{
    protected $connection;
    
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
	}
	
	public function beginTransaction()
	{
		$this->connection->beginTransaction();
	}
	
	public function commit()
	{
		$this->connection->commit();
	}
	
	public function rollback()
	{
		$this->connection->rollback();
	}
	
	public function getMainSupplier($mainSupplier, $accessKey)
	{
		$query = "SELECT CDFORNECED 
				    FROM FORNECEDOR
				   WHERE NRINSJURFORN = :MAIN_SUPPLIER
				     AND DSTOKENINTGECO = :ACCESS_KEY";
				     
		$params = [
			"MAIN_SUPPLIER" => $mainSupplier,
			"ACCESS_KEY" => $accessKey
		];
		
		$std = $this->connection->prepare($query);
        $std->execute($params);

        $result = $std->fetchAll();
        
        if (count($result)) {
        	return $result[0];
        }
        
        return false;
	}
	
	public function getAvailableAFs($mainSupplier, $initialDate, $finalDate, $allDate, $afCode, $state, $status)
	{
		try {
			$query = "SELECT * FROM
						(SELECT
						    FO.NRINSJURFORN AS CNPJ,
						    FO.NMFANTFORN AS NOME,
						    FO.NMRAZSOCFORN AS RAZAO,
						    A.NRAUTOFORN AS NUMERO,
						    A.DTAUTOFORN AS DATACRIACAO,
						    (SELECT DECODE(COUNT(DISTINCT ITEMAUTF.DTPROGENTAF), 1, MAX(DTPROGENTAF), NULL) FROM ITEMAUTF WHERE (A.CDFILDEST = ITEMAUTF.CDFILDEST) AND (A.CDFILLANC = ITEMAUTF.CDFILLANC) AND (A.NRAUTOFORN = ITEMAUTF.NRAUTOFORN)) AS DATAENTREGA,
						    NVL((SELECT DISTINCT 'LIDO' FROM LOGDOWNLOADAF WHERE (A.NRSEQENVIOAFORN = LOGDOWNLOADAF.NRSEQENVIOAFORN) AND (A.CDFORNECED = LOGDOWNLOADAF.CDFORNECED) AND (A.NRAUTOFORN = LOGDOWNLOADAF.NRAUTOFORN) AND (A.CDFILDEST = LOGDOWNLOADAF.CDFILDEST) AND (A.CDFILLANC = LOGDOWNLOADAF.CDFILLANC)), 'DISPONIVEL') AS STATUS,
						    FD.CDFILIAL AS UNIDADEDESTINO,
						    FD.NMFILIAL AS NOMEUNIDADEDESTINO,
						    EF.SGESTADO AS UF,
						    E.IDTCLASSENVAFOR AS TIPO,
						    NVL(A.IDBAIXAAF, 'X') AS SITUACAOBAIXA,
						    PGE.TXMSGFORNECEDOR AS CBC,
						    DE.CDFILIAL AS DADOS_ENTREGA_UNIDADE,
						    DE.NMFILIAL AS DADOS_ENTREGA_NOMEUNIDADE,
						    DE.NRINSJURFILI AS DADOS_ENTREGA_CNPJ,
						    DE.DSENDEFILI AS DADOS_ENTREGA_ENDERECO,
						    DE.NMBAIRFILI AS DADOS_ENTREGA_BAIRRO,
						    DE.NMMUNICIPIO AS DADOS_ENTREGA_MUNICIPIO,
						    DE.SGESTADO AS DADOS_ENTREGA_ESTADO,
						    DE.NRCEPFILI AS DADOS_ENTREGA_CEP,
						    DE.DSPERIENTFIL AS DADOS_ENTREGA_HORARIOENTREGA,
						    DE.NMCONTFILI AS DADOS_ENTREGA_CONTATO,
						    DE.DSEMAILFILI AS DADOS_ENTREGA_EMAIL,
						    DE.NRTELEFILI AS DADOS_ENTREGA_TELEFONE,
						    DE.DSOBSFILIAL AS DADOS_ENTREGA_OBSERVACAO,
						    DECODE(A.IDTIPOEMITAF, '3', C.NRINSJURCLIE, EM.NRINSJUREMPR) AS DADOS_COBRANCA_CNPJ,
						    DECODE(A.IDTIPOEMITAF, '3', EC.DSENDECLIE, EM.DSENDEEMPR) AS DADOS_COBRANCA_ENDERECO,
						    DECODE(A.IDTIPOEMITAF, '3', EC.NMBAIRCLIE, EM.NMBAIREMPR) AS DADOS_COBRANCA_BAIRRO,
						    DECODE(A.IDTIPOEMITAF, '3', MC.NMMUNICIPIO, M.NMMUNICIPIO) AS DADOS_COBRANCA_MUNICIPIO,
						    DECODE(A.IDTIPOEMITAF, '3', EC.SGESTADO, EM.SGESTADO) AS DADOS_COBRANCA_ESTADO,
						    DECODE(A.IDTIPOEMITAF, '3', EC.NRCEPCLIE, EM.NRCEPEMPR) AS DADOS_COBRANCA_CEP,
						    DECODE(A.IDTIPOEMITAF, '3', EC.NMCONTCLIE, EM.NMCONTEMPR) AS DADOS_COBRANCA_CONTATO,
						    DECODE(A.IDTIPOEMITAF, '3', EC.DSEMAILCLIE, EM.DSEMAILEMPRESA) AS DADOS_COBRANCA_EMAIL,
						    DECODE(A.IDTIPOEMITAF, '3', EC.NRTELECLIE, EM.NRTELEFOEMPR) AS DADOS_COBRANCA_TELEFONE,
						    DECODE(PF.CDTIPOFATUR, '003', C.NRINSJURCLIE, FT.NRINSJURIEST) AS DADOS_FATURAMENTO_CNPJ,
						    DECODE(PF.CDTIPOFATUR, '003', EC.DSENDECLIE, FT.DSENDEIEST) AS DADOS_FATURAMENTO_ENDERECO,
						    DECODE(PF.CDTIPOFATUR, '003', EC.NMBAIRCLIE, FT.NMBAIRIEST) AS DADOS_FATURAMENTO_BAIRRO,
						    DECODE(PF.CDTIPOFATUR, '003', MC.NMMUNICIPIO, FT.NMMUNICIPIO) AS DADOS_FATURAMENTO_MUNICIPIO,
						    DECODE(PF.CDTIPOFATUR, '003', EC.SGESTADO, FT.SGESTADO) AS DADOS_FATURAMENTO_ESTADO,
						    DECODE(PF.CDTIPOFATUR, '003', EC.NRCEPCLIE, FT.NRCEPIEST) AS DADOS_FATURAMENTO_CEP,
						    P.CDARVPROD AS PRODUTO_CODCLIENTE, 
						    P.NMPRODUTO AS PRODUTO_NOME, 
						    P.SGUNIDADE AS PRODUTO_UNDCLIENTE, 
						    MA.CDMARCA || ' - ' || MA.NMMARCA AS PRODUTO_MARCA, 
						    IT.DTPROGENTAF AS PRODUTO_DATAENTREGA, 
						    IT.DTPROGENTAF AS PRODUTO_DATAPRIMEIRAUTILIZACAO, 
						    NVL(IT.QTPROGENTAF, 0) AS PRODUTO_QUANTIDADE, 
						    NVL((IT.QTPROGENTAF - NVL(IT.QTEFETENTRAF, 0)), 0) AS PRODUTO_QUANTIDADEPENDENTE, 
						    NVL(IT.VRPREUNIAF, 0) AS PRODUTO_VALORUNITARIO, 
						    NVL(IT.VRDESCITMAF, 0) AS PRODUTO_VALORDESCONTO, 
						    NVL(((IT.QTPROGENTAF * IT.VRPREUNIAF) - IT.VRDESCITMAF), 0) AS PRODUTO_VALORTOTAL,
					      (SELECT LISTAGG(CDPRODUTOXML || ' / ') WITHIN GROUP (ORDER BY CDPRODUTOXML) AS PRODFORNE
                		FROM PRODFORNXML
                	WHERE CDPRODUTO  = IT.CDPRODUTO
                    AND CDFORNECED = A.CDFORNECED) AS PRODUTO_PRODFORNE,
                (SELECT DSPRODUTO
                    FROM PRODFORNCOMP
                  WHERE CDPRODCOMPL  =  IT.CDPRODUTO
                    AND CDFORNECED =  A.CDFORNECED)AS PRODUTO_DESCPRODUTO,
						    A.NRSEQENVIOAFORN,
						    A.CDFILLANC,
						    FO.CDFORNECED
						FROM
						    FORNECEDOR          FO,
						    AUTOFORNE           A,
						    ITEMAUTF            IT,
						    PRODUTO             P,
						    MARCA               MA,
						    FILIAL              FD,
						    ENVIOAFFORN         E,
						    ENDEFILI            EF,
						    ESTADO              ES,
						    PARAMGERALECOMMERCE PGE,
						    CLIENTE             C,
						    CLIENFILIAL         CF,
						    ENDECLIE            EC,
						    MUNICIPIO           M,
						    MUNICIPIO           MC,
						    EMPRESA             EM,
						    PARAMFILIAL         PF,
						    (SELECT 
						        FILIAL.CDFILIAL,
						        FILIAL.NMFILIAL,
						        FILIAL.NRINSJURFILI,
						        ENDEFILI.DSENDEFILI,
						        ENDEFILI.NMBAIRFILI,
						        MUNIFIL.NMMUNICIPIO,
						        ENDEFILI.SGESTADO,
						        ENDEFILI.NRCEPFILI,
						        PARCMPSUP.DSPERIENTFIL,
						        ENDEFILI.NMCONTFILI,
						        ENDEFILI.DSEMAILFILI,
						        ENDEFILI.NRTELEFILI,
						        PARCMPSUP.DSOBSFILIAL
						        FROM
						        FILIAL,
						        ENDEFILI,
						        MUNICIPIO MUNIFIL,
						        PARCMPSUP
						        WHERE (FILIAL.CDFILIAL = ENDEFILI.CDFILIAL)
						        AND ( ENDEFILI.IDTPENDEFILI = DECODE((
						            SELECT
						                IDTPENDEFILI
						            FROM
						                ENDEFILI
						            WHERE
						                IDTPENDEFILI = 'T'
						                AND CDFILIAL = FILIAL.CDFILIAL
						        ), 'T', 'T', 'P') )
						        AND ( ENDEFILI.CDPAIS = MUNIFIL.CDPAIS )
						        AND ( ENDEFILI.SGESTADO = MUNIFIL.SGESTADO )
						        AND ( ENDEFILI.CDMUNICIPIO = MUNIFIL.CDMUNICIPIO )
						        AND ( PARCMPSUP.CDFILIAL = FILIAL.CDFILIAL )
						    ) DE,
						    (SELECT 
						        INSCRESTAD.NMREPRINSEST, INSCRESTAD.DSENDEIEST, INSCRESTAD.NRINSJURIEST, INSCRESTAD.IDTPIJURIEST, 
						        INSCRESTAD.NRCEPIEST, INSCRESTAD.CDMUNICIPIO, MUNICIPIO.NMMUNICIPIO , INSCRESTAD.CDINSCESTA, FILIAL.CDFILIAL, INSCRESTAD.NMBAIRIEST,
						        INSCRESTAD.SGESTADO, INSCRESTAD.DSCOMPENDIES
						        FROM 
						        FILIAL, INSCRESTAD, MUNICIPIO, PAIS, ESTADO
						        WHERE
						        FILIAL.CDINSCESTA = INSCRESTAD.CDINSCESTA AND
						        FILIAL.CDEMPRESA  = INSCRESTAD.CDEMPRESA  AND
						        INSCRESTAD.CDPAIS   = PAIS.CDPAIS AND
						        INSCRESTAD.SGESTADO  = ESTADO.SGESTADO AND   
						        INSCRESTAD.CDMUNICIPIO = MUNICIPIO.CDMUNICIPIO AND
						        PAIS.CDPAIS = MUNICIPIO.CDPAIS AND
						        ESTADO.CDPAIS = PAIS.CDPAIS AND
						        ESTADO.SGESTADO = MUNICIPIO.SGESTADO
						    ) FT
						WHERE
						    ((FO.CDFORNECED = :MAIN_SUPPLIER AND FO.CDFORNECED = A.CDFORNECED)
						        OR 
						        (FO.CDFORNERELACAF = :MAIN_SUPPLIER AND FO.CDFORNECED = A.CDFORNECED))
						    AND (INSTR(UPPER(:STATE_LIST), UPPER(EF.SGESTADO)) <> 0 OR :STATE_LIST = 'ALL')
						    AND (INSTR(:AF_CODE, A.NRAUTOFORN) <> 0 OR :AF_CODE = 'ALL')
						    AND A.NRAUTOFORN = IT.NRAUTOFORN
						    AND A.CDFILLANC = IT.CDFILLANC
						    AND A.CDFILDEST = IT.CDFILDEST
						    AND IT.CDPRODUTO = P.CDPRODUTO
						    AND IT.CDMARCA = MA.CDMARCA
						    AND FD.CDFILCENTCMP = PGE.CDFILIAL(+)
						    AND A.CDFILDEST = CF.CDFILIAL(+)
						    AND CF.CDCLIENTE = C.CDCLIENTE(+)
						    AND CF.CDCLIENTE = EC.CDCLIENTE(+)
						    AND CF.IDCLIENPRIN(+) = 'S'
						    AND EC.IDTPENDECLIE(+) = 'P'
						    AND EC.CDPAIS = MC.CDPAIS(+)
						    AND EC.SGESTADO = MC.SGESTADO(+)
						    AND EC.CDMUNICIPIO = MC.CDMUNICIPIO(+)
						    AND A.CDEMPRESA = EM.CDEMPRESA
						    AND EM.CDPAIS = M.CDPAIS
						    AND EM.SGESTADO = M.SGESTADO
						    AND EM.CDMUNICIPIO = M.CDMUNICIPIO
						    AND FT.CDFILIAL  = A.CDFILDEST
						    AND A.CDFILDEST = PF.CDFILIAL(+)
						    AND A.CDFILDEST = DE.CDFILIAL
						    AND A.NRSEQENVIOAFORN = E.NRSEQENVIOAFORN
						    AND A.CDFORNECED = E.CDFORNECED
						    AND A.CDFILDEST = FD.CDFILIAL
						    AND A.CDFILDEST = EF.CDFILIAL
						    AND EF.CDPAIS = ES.CDPAIS
						    AND EF.SGESTADO = ES.SGESTADO
						    AND (EF.IDTPENDEFILI = DECODE((
						        SELECT
						            ENDEFILI.IDTPENDEFILI
						        FROM
						            ENDEFILI ENDEFILI
						        WHERE
						            ENDEFILI.IDTPENDEFILI = 'T'
						            AND ENDEFILI.CDFILIAL = FD.CDFILIAL
						    ), 'T', 'T', 'P') )
						    AND EXISTS (
						        SELECT
						            1
						        FROM
						            ITEMAUTF
						        WHERE (A.CDFILDEST = ITEMAUTF.CDFILDEST)
						            AND (A.CDFILLANC = ITEMAUTF.CDFILLANC)
						            AND (A.NRAUTOFORN = ITEMAUTF.NRAUTOFORN)
						    )
						ORDER BY
						    CNPJ,
						    NUMERO,
						    PRODUTO_CODCLIENTE)
					  WHERE (DATAENTREGA BETWEEN :INITIAL_DATE AND :FINAL_DATE OR :ALL_DATE = 'ALL')
					    AND (UPPER(STATUS) = UPPER(:STATUS) OR :STATUS = 'ALL')";
						    
			$params = [
				"MAIN_SUPPLIER" => $mainSupplier,
				"INITIAL_DATE" => $initialDate,
				"FINAL_DATE" => $finalDate,
				"ALL_DATE" => $allDate,
				"AF_CODE" => $afCode,
				"STATE_LIST" => $state,
				"STATUS" => $status
			];
			
			$std = $this->connection->prepare($query);
	        $std->execute($params);
	
	        $result = $std->fetchAll();
	        
	    	return $result;
		} catch(\Exception $e) {
			throw new \Exception("Erro ao consultar as AFs.");
		}
	}
	
	public function setLogDownloadAF($params)
	{
		try {
			$query = "INSERT INTO LOGDOWNLOADAF
	                	 (NRSEQDOWNLOADAF,
	                	 NRSEQENVIOAFORN,
	                	 CDFORNECED,
	                	 NRAUTOFORN,
	                	 CDFILDEST,
	                	 CDFILLANC,
	                	 DSLOGINECO,
	                	 DTDOWNLOAD)
	                  VALUES 
	                	(:NRSEQDOWNLOADAF,
	                	:NRSEQENVIOAFORN,
	                	:CDFORNECED,
	                	:NRAUTOFORN,
	                	:CDFILDEST,
	                	:CDFILLANC,
	                	:DSLOGINECO,
	                	SYSDATE)";
	        
	        $std = $this->connection->prepare($query);
	        $std->execute($params);
		} catch(\Exception $e) {
			throw new \Exception("Erro ao inserir o log de AFs lidas.");
		}
	}
}